Class: Proxy::DHCP::KeaApi::Plugin
- Inherits:
-
Provider
- Object
- Provider
- Proxy::DHCP::KeaApi::Plugin
- Defined in:
- lib/smart_proxy_dhcp_kea_api/dhcp_kea_api_plugin.rb
Overview
The main plugin class for the ‘dhcp_kea_api` provider. This class serves as the entry point for the Foreman Smart Proxy to load and configure the plugin. It defines the plugin’s name, version, dependencies on other modules, default settings, and hooks into the dependency injection framework.